<?php declare(strict_types=1);

namespace Everyware\Plugin\CacheInvalidator;

use OcUtilities;

/**
 * Class EventHandler
 * @package Everyware\Plugin\CacheInvalidator
 */
class EventHandler
{
    public function __construct()
    {

        /**
         * Triggers on post save, check if we should update page data.
         */
        add_action('save_post', function ($postId) {

            $postType = get_post_type($postId);
            if ($postType === 'page' && get_post_status($postId) !== 'trash') {
                $this->pageHasBeenUpdated($postId);
            }

            return;
        });

        /**
         * Triggers on post sent to trash, check if we should update page data.
         */
        add_action('trashed_post', function ($postId) {
            $this->postHasBeenDeleted($postId);

            return;
        });

        /**
         * Triggers on post deleted, check if we should update page data.
         */
        add_action('deleted_post', function ($postId) {
            $this->postHasBeenDeleted($postId);

            return;
        });

        /**
         * Triggers on board updated, update page data.
         */
        add_action('board_changed_hook', function ($pageId) {
            $this->pageHasBeenUpdated($pageId);
        });

        /**
         * Triggers on widget update, update data for pages where widget is active.
         */
        add_filter('widget_update_callback', function ($instance, $new, $old, $obj) {

            if (method_exists($obj, 'get_oc_query')) {

                $oldQuery = PageQueries::getQueryFromWidgetInstance($obj->id, $old);
                $newQuery = PageQueries::getQueryFromWidgetInstance($obj->id, $new);

                if ($newQuery !== $oldQuery) {

                    $dataStorage = new DataStorage();
                    $lcc = new Lcc();
                    $boards = $this->getBoardsForWidget($obj->id);

                    foreach ($boards as $board) {
                        $boardPageIds = $this->getPagesConnectedToBoard($board['id']);
                        if (count($boardPageIds) > 0) {
                            foreach ($boardPageIds as $boardPageId) {

                                $pageItems = $dataStorage->getItemsByPageId($boardPageId);
                                foreach ($pageItems as $item) {
                                    if ($oldQuery === $item->oc_query) {
                                        $lcc->deregisterListener($item->listener_id);
                                        $dataStorage->deleteItem($item->id);
                                    }
                                }

                                $listenerId = $lcc->registerListener($newQuery);
                                $dataStorage->saveItem(new QueryItem($newQuery, $boardPageId, 'widget', $listenerId));
                            }
                        }
                    }
                }
            }

            return $instance;
        }, 10, 4);

        /**
         * Triggers on oc update for articles.
         */
        add_action('oc_push_article_update', function ($data) {
            if (isset($data['uuid'])) {
                $postId = OcUtilities::get_article_post_id_by_uuid($data['uuid']);
                if (isset($postId)) {
                    LedgeInvalidator::invalidateUrl(get_permalink($postId));
                }
            }
        });

        /**
         * Triggers on update for lists.
         */
        add_action('oc_push_list_update', function ($data) {
            if (isset($data['uuid'])) {
                $page_ids = $this->getPageIdsForOcList($data['uuid']);
                foreach ($page_ids as $page_id) {
                    $this->pageHasBeenUpdated($page_id);
                }
            }
        });
    }

    /**
     * Post has been deleted, update query data for pages.
     *
     * @param $post_id
     */
    private function postHasBeenDeleted($post_id): void
    {

        $postType = get_post_type($post_id);
        if ($postType === 'page') {
            $this->pageHasBeenDeleted($post_id);
        } elseif ($postType === 'board') {

            $boardPageIds = $this->getPagesConnectedToBoard($post_id);
            if ($boardPageIds !== false) {
                foreach ($boardPageIds as $boardPageId) {
                    $this->pageHasBeenDeleted($boardPageId);
                }
            }
        }
    }

    /**
     * Page has been updated, update query data for page.
     *
     * @param $page_id
     */
    private function pageHasBeenUpdated($page_id): void
    {

        $lcc = new Lcc();
        $dataStorage = new DataStorage();
        $existingPageItems = $dataStorage->getItemsByPageId($page_id);

        $pageQueries = new PageQueries();
        $updatedPageItems = $pageQueries->getPageQueries($page_id);
        $newPageItems = [];

        foreach ($updatedPageItems as $query) {

            if (is_array($query) && count($query) >= 3) {
                $newPageItems[] = new QueryItem($query[0], $query[1], $query[2]);
            }
        }

        foreach ($newPageItems as $item) {
            $removeItem = null;
            foreach ($existingPageItems as $existingItem) {

                if ($item->oc_query === $existingItem->oc_query && $item->type === $existingItem->type) {
                    $item->id = $existingItem->id;
                    $item->listener_id = $existingItem->listener_id;
                }
            }
        }

        foreach ($newPageItems as $newItem) {

            if ( ! isset($newItem->id) || $newItem->id === 0) {

                $listener_id = $lcc->registerListener($newItem->oc_query);
                if ($listener_id !== false) {
                    $newItem->listener_id = $listener_id;
                    $dataStorage->saveItem($newItem);
                }
            }
        }

        foreach ($existingPageItems as $existingItem) {

            $itemExists = false;
            foreach ($newPageItems as $newItem) {

                if ($existingItem->oc_query === $newItem->oc_query && $existingItem->type === $newItem->type) {
                    $itemExists = true;
                }
            }

            if ( ! $itemExists) {
                $lcc->deregisterListener($existingItem->listener_id);
                $dataStorage->deleteItem($existingItem->id);
            }
        }
    }

    /**
     * Page has been deleted, delete query data for page.
     *
     * @param $pageId
     */
    private function pageHasBeenDeleted($pageId): void
    {

        $dataStorage = new DataStorage();
        $pageItems = $dataStorage->getItemsByPageId($pageId);

        if ($pageItems !== false) {

            $lcc = new Lcc();
            foreach ($pageItems as $item) {
                $lcc->deregisterListener($item->listener_id);
            }

            $dataStorage->deleteItemsByPageId($pageId);
        }
    }

    /**
     * Get boards where widget id is active.
     *
     * @param $widgetId
     *
     * @return array
     */
    private function getBoardsForWidget($widgetId): array
    {

        $boardsArr = [];
        $board_args = [
            'posts_per_page' => -1,
            'post_type' => 'everyboard',
            'post_status' => 'any'
        ];

        $boards = get_posts($board_args);
        $boards_data = [];

        foreach ($boards as $board) {
            $json = get_post_meta($board->ID, 'everyboard_json', true);
            $boards_data[$board->ID]['title'] = $board->post_title;
            $boards_data[$board->ID]['json'] = $json;
        }

        array_walk($boards_data, function ($board, $key) use (&$boardsArr, $widgetId) {
            if (false !== strpos($board['json'], $widgetId)) {
                $boardsArr[] = [
                    'id' => $key,
                    'title' => $board['title']
                ];
            }
        });

        return $boardsArr;
    }

    /**
     * Get page ids where the list uuid is used.
     *
     * @param $listUuid
     *
     * @return array
     */
    private function getPageIdsForOcList($listUuid): array
    {

        $boardArgs = [
            'posts_per_page' => -1,
            'post_type' => 'everyboard',
            'post_status' => 'any'
        ];

        $boardPosts = get_posts($boardArgs);
        $boardsData = [];

        foreach ($boardPosts as $board) {
            $json = get_post_meta($board->ID, 'everyboard_json', true);
            $boardsData[$board->ID]['json'] = $json;
        }

        $boardIds = [];
        foreach ($boardsData as $id => $board) {
            if (false !== strpos($board['json'], $listUuid)) {
                $boardIds[] = $id;
            }
        }

        if (count($boardIds) === 0) {
            return [];
        }

        /* the inner empty array covers cases when no loops were made */
        $pageIds = [[]];
        foreach ($boardIds as $boardId) {
            $pageIds[] = $this->getPagesConnectedToBoard($boardId);
        }

        $pageIds = array_merge(...$pageIds);

        return $pageIds;
    }

    /**
     * Get pages that are connected to given board id.
     *
     * @param $boardId
     *
     * @return array
     */
    private function getPagesConnectedToBoard($boardId): array
    {

        $args = [
            'post_type' => 'page',
            'meta_query' => [
                [
                    'key' => 'everyboard_id',
                    'value' => $boardId,
                    'compare' => '='
                ]
            ]
        ];

        $posts = get_posts($args);
        $returnArr = [];
        foreach ($posts as $page) {
            $returnArr[] = $page->ID;
        }

        return $returnArr;
    }
}
